<?php

namespace FiloBlu\CaptchaV3\Helper;

use Magento\Framework\App\Helper\AbstractHelper;
use Magento\Store\Model\ScopeInterface;

class Config extends AbstractHelper
{
    /**
     * Configuration path
     */
    const CONFIG_PATH_GENERAL_ENABLE_MODULE = 'filoblu_captcha/general/enabledCaptcha';
    const CONFIG_PATH_GENERAL_SITE_KEY = 'filoblu_captcha/general/captchaSiteKey';
    const CONFIG_PATH_GENERAL_SECRET_KEY = 'filoblu_captcha/general/captchaSecretKey';
    const CONFIG_PATH_GENERAL_TRESHOLD = 'filoblu_captcha/general/captchaTreshold';
    const CONFIG_PATH_GENERAL_ALLOWED_URLS = 'filoblu_captcha/general/captchaUrls';
    const CONFIG_PATH_DISPLAY_ALLOWED_SELECTORS = 'filoblu_captcha/general/captchaSelectors';


    /**
     * @param $string
     *
     * @return mixed
     */
    public function stringValidationAndCovertInArray($string)
    {
        $validate = function ($urls) {
            return preg_split('|\s*[\r\n]+\s*|', $urls, -1, PREG_SPLIT_NO_EMPTY);
        };

        return $validate($string);
    }

    /**
     * Is google recaptcha enable
     *
     * @param string $scopeType
     * @param mixed $scopeCode
     *
     * @return bool
     */
    public function hasEnable(
        $scopeType = ScopeInterface::SCOPE_WEBSITE,
        $scopeCode = null
    ) {
        return $this->scopeConfig->isSetFlag(
            self::CONFIG_PATH_GENERAL_ENABLE_MODULE,
            $scopeType,
            $scopeCode
        );
    }

    /**
     * Return Captcha Key
     *
     * @param string $scopeType
     * @param mixed $scopeCode
     *
     * @return string
     */
    public function getSiteKey(
        $scopeType = ScopeInterface::SCOPE_WEBSITE,
        $scopeCode = null
    ) {
        return $this->scopeConfig->getValue(
            self::CONFIG_PATH_GENERAL_SITE_KEY,
            $scopeType,
            $scopeCode
        );
    }

    /**
     * Return Secret Key
     *
     * @param string $scopeType
     * @param mixed $scopeCode
     *
     * @return string
     */
    public function getSecretKey(
         $scopeType = ScopeInterface::SCOPE_WEBSITE,
        $scopeCode = null
    ) {
        return $this->scopeConfig->getValue(
            self::CONFIG_PATH_GENERAL_SECRET_KEY,
            $scopeType,
            $scopeCode
        );
    }

    /**
     * Return Treshold
     *
     * @param string $scopeType
     * @param mixed $scopeCode
     *
     * @return string
     */
    public function getTreshold(
         $scopeType = ScopeInterface::SCOPE_WEBSITE,
        $scopeCode = null
    ) {
        return $this->scopeConfig->getValue(
            self::CONFIG_PATH_GENERAL_TRESHOLD,
            $scopeType,
            $scopeCode
        );
    }

    /**
     * @param $path
     * @param null $storeId
     * @param string $scope
     *
     * @return mixed
     */
    public function getConfigValueByPath(
        $path,
        $storeId = null,
        $scope = ScopeInterface::SCOPE_WEBSITE
    ) {
        return $this->scopeConfig->getValue($path, $scope, $storeId);
    }

    /**
     * @param string $scopeType
     * @param null $scopeCode
     *
     * @return array
     */
    public function getCaptchaUrls(
        $scopeType = ScopeInterface::SCOPE_WEBSITE,
        $scopeCode = null
    ) {
        $urls = $this->scopeConfig->getValue(
            self::CONFIG_PATH_GENERAL_ALLOWED_URLS,
            $scopeType,
            $scopeCode
        ) ?: '';
        $urls = trim($urls);

        return $urls ? $this->stringValidationAndCovertInArray($urls) : [];
    }

    /**
     * @param string $scopeType
     * @param null $scopeCode
     *
     * @return array
     */
    public function getCaptchaSelectors(
        $scopeType = ScopeInterface::SCOPE_WEBSITE,
        $scopeCode = null
    ) {
        $urls = $this->scopeConfig->getValue(
            self::CONFIG_PATH_DISPLAY_ALLOWED_SELECTORS,
            $scopeType,
            $scopeCode
        ) ?: '';
        $urls = trim($urls);

        return $urls ? $this->stringValidationAndCovertInArray($urls) : [];
    }
}
